#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <AT91SAM7S.h>
#include <AT91SAM7S_BITDEFINE.h>
#include <vic.h>

#include <dx.h>

#define __UDP__
#include "share\us.c"

#define _FD_PB  (!(PIOA.PDSR & ((uint32_t)1 << 4)))

typedef struct {
  uint8_t   id;
  uint8_t   baud;
  uint16_t  modelno;
  int       ud;
} TDX;

char t1[100],r1[100];

void main (void) {

#define SPD  1023
//#define RAMP 20
#define RAMP 100

  void fin(void) {
    if (_FD_PB) {
      DX_WriteWordData (1, ADDRESS_GOAL_SPEED, 0);
      DX_WriteWordData (2, ADDRESS_GOAL_SPEED, 0);
      DX_WriteByteData (1, ADDRESS_TORQUE_ENABLE, 0);
      DX_WriteByteData (2, ADDRESS_TORQUE_ENABLE, 0);
      udp_disconnect ();
      DX_wait (3000);
      RSTC.CR = 0xA5000000 | _RSTC_PROCRST | _RSTC_PERRST | _RSTC_EXTRST;
    }
  }

  void spdset (int s1,int s2) {
    static int o1 = 0, o2 = 0;

    if (s1 > 1023) s1 = 1023;
    else if (s1 < -1023) s1 = -1023;
    if (s2 > 1023) s2 = 1023;
    else if (s2 < -1023) s2 = -1023;

    if ((o1 != s1) || (o2 != s2)) {
      if (o1 > s1) {
        o1-=RAMP;
        if (o1 < s1) o1 = s1;
      } else if (o1 < s1) {
        o1+=RAMP;
        if (o1 > s1) o1 = s1;
      }
      if (o2 > s2) {
        o2-=RAMP;
        if (o2 < s2) o2 = s2;
      } else if (o2 < s2) {
        o2+=RAMP;
        if (o2 > s2) o2 = s2;
      }
      DX_WriteWordData (1, ADDRESS_GOAL_SPEED, abs(o1) | ((o1 < 0) ? (1 << 10) : 0));
      DX_WriteWordData (2, ADDRESS_GOAL_SPEED, abs(o2) | ((o2 < 0) ? (1 << 10) : 0));
    }
  }

  bool automode = false;
  char c;
  int i, t;
  int s1 = 0,s2 = 0;
  uint16_t dat1 = 0, old1 = 0;
  uint8_t dat2 = 0, old2 = 0;
  int turntime = 0, turndir = 0;



  enableIRQ ();

  _us_init (_BAUDRATE, txb, sizeof (txb), rxb, sizeof (rxb));
  us1_pdc_init (460800, t1, sizeof (t1), r1, sizeof (r1));
  DX_Init (1000000, 0, NULL, NULL);

  for (i = 0; i < 60; i++) {
    DX_ToneOn (i / 12, i % 12);
    DX_wait (10);
  }
  DX_ToneOff();

  DX_WriteByteData (1, ADDRESS_LOCK, 1);
  DX_WriteByteData (2, ADDRESS_LOCK, 1);
  DX_WriteByteData (1, ADDRESS_TORQUE_ENABLE, 1);
  DX_WriteByteData (2, ADDRESS_TORQUE_ENABLE, 1);

  while (1) {
    if (_us_rx_buff ()) {
      c = toupper((int)_us_getc ());
      _us_rx_purge ();
    } else if (us1_pdc_rx_buff ()) {
      c = toupper((int)us1_pdc_getc ());
      us1_pdc_rx_purge ();
    } else
      c = 0;

    switch (c) {
      case 'A':
        automode = false;
        turntime = DX_ElapsedTime;
        s1 = s2 = 0;
        break;

      case 'B':
        automode = false;
        turntime = DX_ElapsedTime;
        s1 = -SPD;
        s2 = SPD;
        break;
      case 'C':
        automode = false;
        turntime = DX_ElapsedTime;
        s1 = SPD;
        s2 = -SPD;
        break;
      case 'D':
        automode = false;
        turntime = DX_ElapsedTime;
        s1 = SPD;
        s2 = SPD;
        break;
      case 'E':
        automode = false;
        turntime = DX_ElapsedTime;
        s1 = -SPD;
        s2 = -SPD;
        break;

      case 'Z':
        automode = true;
        turntime = DX_ElapsedTime;
        break;
    }

    if (DX_ReadWordData (100, 26, &dat1)) {
      dat1 = dat1 >> 5;
      if (dat1 > 25) dat1 = 25;
    }
    if (DX_ReadByteData (100, 42, &dat2)) {
      dat2 = (dat2 >> 1) & 3;
    }
    if (old1 != dat1) {
      _us_printf ("%c", 'A' + dat1);
      us1_pdc_printf ("%c", 'A' + dat1);
    }
    if (old2 !=dat2) {
      _us_printf ("%1d", dat2);
      us1_pdc_printf ("%1d", dat2);
    }
    old1 = dat1;
    old2 = dat2;

    if (automode) {
      //------------------------------------------
      //----- ^[wȂꍇ
      //------------------------------------------
      if (turndir == 0) {
        switch (old2) {
          case 0:
            if (dat1 < 18) {
              s1 = -(SPD - (dat1 * 10));
              s2 =  (SPD - (dat1 * 10));
              turndir = 0;
              turntime = DX_ElapsedTime;
            } else {
              turndir = 2;
              turntime = DX_ElapsedTime + 10000;
            }
            break;
          case 1:
            turntime = DX_ElapsedTime + 20000;
            turndir = 2;
            break;
          case 2:
          case 3:
            turntime = DX_ElapsedTime + 20000;
            turndir = 1;
            break;
        }
      //------------------------------------------
      //----- ^[wꍇ
      //------------------------------------------
      } else {
        t = turntime - (int)DX_ElapsedTime;
        // ^CAbv܂1bȏ゠ -> obN
        if (t >= 10000) {
          s1 = SPD;
          s2 = -SPD;
        // ^CAbv܂1b -> 
        } else if ((10000 > t) && (t > 500)){
          switch (turndir) {
            case 1:
              s1 = SPD;
              s2 = SPD;
              break;
            case 2:
              s1 = -SPD;
              s2 = -SPD;
              break;
          }
        // ^CAbv -> ^[wLZ
        } else {
          turndir = 0;
          turntime = DX_ElapsedTime;
        }
      }
    }

    spdset (s1, s2);

    fin();
    DX_wait (10);
  }
}
